#ifndef ODB_H
#define ODB_H

/*
 * Copyright (c) Abraham vd Merwe <abz@blio.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *	  notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *	  notice, this list of conditions and the following disclaimer in the
 *	  documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the author nor the names of other contributors
 *	  may be used to endorse or promote products derived from this software
 *	  without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdlib.h>

#include <tinysnmp/tinysnmp.h>
#include <tinysnmp/compat/inttypes.h>

/*
 * Destroy an ObjectID database. When this function returns, the
 * database is in exactly the same state as when initialized.
 */
extern void odb_destroy ();

/*
 * Add a new ObjectID to the ObjectID database. Returns 0 if
 * successful, -1 otherwise. Check errno to see what error
 * occurred. The following errors is possible:
 *
 *       ENOMEM     Out of memory
 *       EEXIST     An ObjectID like this (or superset of) already exist
 */
#define odb_add(oid,value) odb_add_stub(__FILE__,__LINE__,__FUNCTION__,oid,value)
extern int odb_add_stub (const char *filename,int line,const char *function,const uint32_t *oid,const snmp_value_t *value);

/*
 * Update the value of an existing branch. Returns 0 if successful,
 * or -1 if the ObjectID doesn't exist.
 */
extern int odb_update (const uint32_t *oid,snmp_value_t *value);

/*
 * Find an ObjectID in the ObjectID database. Return the value of
 * an existing ObjectID in the database if successful, or NULL if
 * the ObjectID doesn't exist.
 */
extern const snmp_value_t *odb_find (const uint32_t *oid);

/*
 * Find the ObjectID in the ObjectId database that lexographically
 * succeeds the specified one or NULL if one doesn't exist. The
 * specified ObjectID doesn't need to exist in the database and can
 * also be a partial ObjectID. Check errno to see what error occurred.
 * The following errors is possible:
 *
 *       ENOMEM     Out of memory
 *       ENOENT     Specified ObjectID does not lexographically
 *                  preceed an existing ObjectID
 */
extern snmp_next_value_t *odb_find_next (const uint32_t *oid);

/*
 * Remove an ObjectID subtree from the ObjectID database. Returns 0
 * if successful, or -1 if the ObjectID subtree doesn't exist.
 */
extern int odb_remove (const uint32_t *oid);

/*
 * Display an ObjectID database. Returns 0 if successful, or -1 if some
 * error occurred. Check errno to see what error occurred.
 */
#define odb_show(level) odb_show_stub(__FILE__,__LINE__,__FUNCTION__,level)
extern int odb_show_stub (const char *filename,int line,const char *function,int level);

#endif	/* #ifndef ODB_H */
